# Importing libraries
import cv2
import numpy as np
import matplotlib.pyplot as plt

#class ImageSegmentation
class ImageSegmentation:
    def __init__(self, image_path):
        self.image_path = image_path

    def read_image(self):
        # Reading the image using OpenCV
        self.image = cv2.imread(self.image_path)

    def segment_image(self, num_segments):
        # Converting the image from BGR to RGB (OpenCV uses BGR by default)
        image_rgb = cv2.cvtColor(self.image, cv2.COLOR_BGR2RGB)

        # Reshaping the image to a 2D array of pixels (rows represent pixels, columns represent color channels)
        pixels = image_rgb.reshape(-1, 3).astype(np.float32)

        # k-means clustering on the pixels
        criteria = (cv2.TERM_CRITERIA_EPS + cv2.TERM_CRITERIA_MAX_ITER, 100, 0.2)
        _, labels, centers = cv2.kmeans(pixels, num_segments, None, criteria, 10, cv2.KMEANS_RANDOM_CENTERS)

        # Converting the centers back to 8-bit integers (RGB color values)
        centers = np.uint8(centers)

        # Mapping the labels to the center values to get the segmented image
        self.segmented_image = centers[labels.flatten()]
        self.segmented_image = self.segmented_image.reshape(image_rgb.shape)

    def detect_edges(self):
        # Converting the segmented image to grayscale
        gray_image = cv2.cvtColor(self.segmented_image, cv2.COLOR_RGB2GRAY)

        # Performing Canny edge detection
        edges = cv2.Canny(gray_image, 100, 200)

        # Combining the edges with the segmented image
        self.segmented_image = cv2.cvtColor(self.segmented_image, cv2.COLOR_RGB2BGR)
        self.segmented_image[edges != 0] = [0, 255, 0]  # Set the edge pixels to green

    def display_images(self):
        # Display the original image, segmented image, and edges using matplotlib
        plt.figure(figsize=(15, 5))

        plt.subplot(1, 3, 1)
        plt.imshow(cv2.cvtColor(self.image, cv2.COLOR_BGR2RGB))
        plt.title('Original Image')
        plt.axis('off')

        plt.subplot(1, 3, 2)
        plt.imshow(self.segmented_image)
        plt.title('Segmented Image')
        plt.axis('off')

        plt.subplot(1, 3, 3)
        plt.imshow(cv2.cvtColor(self.segmented_image, cv2.COLOR_BGR2RGB))
        plt.title('Edges Detected')
        plt.axis('off')

        plt.show()

# Reading  the path of image
path_to_image=input("Enter the path of your image: ")
def Imgproc(path_to_image):

    #  number of segments for k-means clustering
    num_segments = int(input("Enter number of segments: "))
    #  Image Segmentation object and perform the   segmentation
    segmentation = ImageSegmentation(path_to_image)
    segmentation.read_image()
    segmentation.segment_image(num_segments)

    # Detecting edges and displaying the images
    segmentation.detect_edges()
    segmentation.display_images()

